# Bash commands

The tool `run_bash_command` allows you to run the following commands:
- `kubectl get|describe|logs|top|events [options]`
- `aws <service> <operation> [options]` (read-only)
- `az|docker|helm|argocd [options]`
- `grep|cut|sort|uniq|head|tail|wc|tr|base64|jq|sed [options]`

Commands can be piped with `|`. No `&&` support.


The tool `kubectl_run_image` will run an image:
- `kubectl run <name> --image=<image> --rm --attach --restart=Never --i --tty -- <command>`
